#
# "@(#) $Id: CDLibrary.py,v 1.6 2004/07/21 17:54:18 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from Library import *
from CDTrack import *
from LMusicPlayer import *
from LsongsDevices import *
import cdinfo
import os
import warnings

def trim(s):
	if s==None: return None
	return s.strip()
#
# the singleton instance of the CD library
#
_CDLibrary = None
cdrom = None # persistent instance of pyxine.stream to work around xine_dispose() bug

class CDLibrary(Library):
	def __init__(self):
		Library.__init__(self)
		self.name = "CDROM"
		self.dirty = True
		self.addNewPlaylist("CD")
		self.info = None

	#
	# see if we have info in cache, if not, load from online database
	#
	def loadInfo(self):
		self.disc_id = cdinfo.disc_id()
		if self.disc_id!=None:
			if not self.fetchCachedTracks(self.disc_id):
				self.fetchTracks(self.disc_id)
			return True
		return False

	#
	# compute the path the CDDB cache directory
	#
	def cachePath(self):
		library = Library.mainLibrary()
		path = os.path.join(library.libraryPath(),".CDCache")
		library.ensureDirectory(path)
		return path
	
	#
	# compute the path to the file for this CDDB entry
	#
	def cachePathForID(self,disc_id):
		hexid = "%08lx" % disc_id[0]
		return os.path.join(self.cachePath(),hexid)

	#
	# check to see if this CD is in the cache
	#
	def fetchCachedTracks(self,disc_id):
		path = self.cachePathForID(disc_id)
		if os.path.exists(path):
			file = open(path,"r")
			s = file.read()
			self.info = eval(s)
			file.close()
			return True
		return False

	#
	# write this CD's CDDB info into the cache
	#
	def writeCachedTracks(self,disc_id,read_info):
		path = self.cachePathForID(disc_id)
		file = open(path,"w")
		file.write(str(read_info))
		file.close()

	#
	# fetch track info about this CD from the CDDB/FreeDB service
	# if we get it, save it in the cache
	#
	def fetchTracks(self,disc_id):
		self.info = cdinfo.getCookedCDDBInfo(disc_id)
		if self.info!=None:
			self.writeCachedTracks(disc_id,self.info)
			return True
		return False

	def trackAfter(self,track):
		trackNum = track.trackNum+1
		if trackNum>len(self._tracks):
			trackNum = 1
		return self.trackWithTrackNum(trackNum)
	
	def trackBefore(self,track):
		trackNum = track.trackNum-1
		if trackNum<1:
			trackNum = len(self._tracks)
		return self.trackWithTrackNum(trackNum)
	
	def trackWithTrackNum(self,trackNum):
		for trackKey in self._tracks.keys():
			track = self._tracks[trackKey]
			if track.trackNum==trackNum:
				return track
		return None

	def writeBinaryFile(self):
		pass
	
	def writeXMLFile(self):
		pass

	#
	# return the singleton instance of the library corresponding to the CD
	#
	def static_singleton():
		global _CDLibrary
		if _CDLibrary==None:
			_CDLibrary = CDLibrary()
		return _CDLibrary

	singleton = staticmethod(static_singleton)
	
	def static_clobber():
		CDLibrary.singleton().dirty = True
	
	clobber = staticmethod(static_clobber)

