#
# "@(#) $Id: CDTrack.py,v 1.13 2004/08/18 20:19:51 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from Track import *
from LMusicPlayer import *
from LsongsDevices import *
import pyid3lib
import time, re

_cdromStream = None

def trim(s):
	if s==None: return None
	return s.strip()

def cleanup(s):
	return re.sub(r"/",r"-",s)
	
class CDTrack(Track):
	def __init__(self,library = None):
		Track.__init__(self,library)
		self.kind = "CD Track"
		self.location = "cdda:/1"
		self.image = None
		self.isCD = True
		self.ripStatus = "idle"
		self.bitRate = 1500000
		self.sampleRate = 44100
		self.format = "PCM"
		self.channels = 2
		self.mode = "Stereo"

	def play(self,playlist = None):
		LMusicPlayer.playTrack(self,self._library,playlist)

	def pause(self):
		LMusicPlayer.pause()

	def loadFromCD(self,trackNum):
		self.trackNum = trackNum
		self.location = "cdda:/%d" % trackNum
		self.originalLocation = self.location
		info = self._library.info
		if info:
			playlists = info['Playlists']
			playlist = playlists[len(playlists)-1]
			track = playlist[trackNum-1]
			#print data
			self.trackCount = track['Track Count']
			self.title = track['Title']
			self.totalTime = track['Total Time']
			try: self.album = track['Album']
			except: pass
			try: self.artist = track['Artist']
			except: pass
			try: self.genre = track['Genre']
			except: pass
			try: self.year = track['Year']
			except: pass
			try: self.compilation = track['Compilation']
			except: pass
		else:
			self.title = "Track %02d" % self.trackNum
			disc_id = self._library.disc_id
			#print disc_id
			start = disc_id[1+trackNum]*1000/75
			self.trackCount = disc_id[1]
			if trackNum==self.trackCount:
				end = disc_id[2+trackNum]*1000
			else:
				end = disc_id[2+trackNum]*1000/75
			#print start,end
			self.totalTime = end-start
		self.size = self.totalTime*1500/8
		
##~ 	def xloadFromCD(self,trackNum):
##~ 		global _cdromStream
##~ 		if _cdromStream==None:
##~ 			xine = LMusicPlayer.xine()
##~ 			ao = xine.open_audio_driver(LsongsDummyAudioDevice)
##~ 			_cdromStream = xine.stream_new(ao)
##~ 		self.trackNum = trackNum
##~ 		self.location = "cdda:/%d" % trackNum
##~ 		self.originalLocation = self.location
##~ 		_cdromStream.open(self.location)
##~ 		#print _cdromStream.title
##~ 		#print _cdromStream.artist
##~ 		self.title = trim(_cdromStream.title)
##~ 		self.genre = trim(_cdromStream.genre)
##~ 		self.artist = trim(_cdromStream.artist)
##~ 		self.album = trim(_cdromStream.album)
##~ 		if self.title==None or len(self.title)==0:
##~ 			self.title = "Track %d" % self.trackNum
##~ 		else:
##~ 			self.title = cleanup(self.title)
##~ 		(percent,position,length) = _cdromStream.get_pos_length()
##~ 		self.totalTime = long(length*1000)
##~ 		_cdromStream.close()

	def copyTagsInto(self,filePath):
		dst = pyid3lib.tag(filePath)
		if self.title:
			dst.title = self.title.encode('latin-1')
		if self.artist:
			dst.artist = self.artist.encode('latin-1')
		if self.album:
			dst.album = self.album.encode('latin-1')
		if self.year:
			dst.year = self.year
		if self.contentType:
			dst.contenttype = self.contentType
		self.mediatype = "CD"
		#if self.compilation:
		#	dst.compilation = self.compilation
		dst.track = (self.trackNum,self.trackCount)
		dst.encodedby = "Lsongs 1.0"
		dst.date = time.strftime("%Y-%m-%dT%H:%M:%S",time.gmtime(time.time()))
		dst.update()

	def setRipStatus(self,status):
		if status['Status'] in ['Begin','Ripping','Encoding']:
			self.ripStatus = 'ripping'
		elif status['Status']=='EndTrack':
			if status.get('Interrupted',False):
				self.ripStatus = 'idle'
			elif status['Source']=='Encoder':
				self.ripStatus = 'ripped'
		self.emit(PYSIGNAL("ripStatus"),(self,status))
