#
# "@(#) $Id: LDellTrackView.py,v 1.5 2004/07/21 17:54:18 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from kdeemul import *
from LDellTrackDrag import *
from LDellTrackViewItem import *
from DellLibrary import *
from DellNJB import *
from LTrackView import *
from LSettings import *
#from LMusicRipper import *

class LDellTrackView(LTrackView,KListView):
	def __init__(self,container):
		LTrackView.__init__(self)
		KListView.__init__(self,container,"audiocd")
		self.settings = LSettings.settings()
		self.library = DellLibrary.singleton()
		self.playlist = self.library.playlistWithName("NJB")
		self.setSelectionMode(QListView.Extended)
		self.setAllColumnsShowFocus(True)
		self.loadHeaders()
		c = QColor(237,243,254)
		self.setAlternateBackground(c)
		self.loadTrackItems()
		self.njb = DellNJB.singleton()
		#QObject.connect(self.njb,PYSIGNAL("dellInserted"),self.refresh)
		QObject.connect(self.library.qObject(),PYSIGNAL("readTracks"),self.refresh)
		QObject.connect(self,SIGNAL('selectionChanged()'),self.doSelectionChanged)

	def loadHeaders(self):
		headers = [" ",i18n("Song Name"),i18n("Time"),i18n("Artist"),i18n("Album"),i18n("Genre")]
		widths = [25,220,40,120,180,80]
		for index in xrange(0,len(headers)):
			self.addColumn(headers[index],widths[index])
		self.setShowSortIndicator(True)
		self.setFullWidth(True)
		sortColumn = self.settings.get('DDJ Library Sort Column',3)
		sortAscending = self.settings.get('DDJ Library Sort Ascending',True)
		self.setSorting(sortColumn,sortAscending)
		QObject.connect(self.header(),SIGNAL("indexChange(int,int,int)"),self.columnIndexChanged)
		QObject.connect(self.header(),SIGNAL("sizeChange(int,int,int)"),self.columnSizeChanged)
		self.loadHeaderSizes('DDJ Library Header Widths')
		self.loadHeaderOrdering('DDJ Library Header Map')

	def loadTrackItem(self,track):
		item = LDellTrackViewItem(self,track)

	def anyItemsSelected(self):
		item = self.firstChild()
		while item:
			if item.isSelected():
				return True
			item = item.itemBelow()
		return False

	def firstSelectedItem(self):
		item = self.firstChild()
		while item:
			if item.isSelected():
				return item
			item = item.itemBelow()
		return None

	def loadTrackItems(self):
		self.clear()
		tracks = self.library.tracks()
		for trackKey in tracks.keys():
			track = tracks[trackKey]
			if self.matchesSearch(track):
				self.loadTrackItem(tracks[trackKey])
		self.emitCurrentEnabledSortedTracks()
		self.emit(PYSIGNAL("changed"),(self,None))

	def refresh(self):
		self.loadTrackItems()
	
	def contentsMouseDoubleClickEvent(self,event):
		pos = self.contentsToViewport(event.pos())
		item = self.itemAt(pos)
		if item:
			item.contentsMouseDoubleClickEvent(event)

	def selectedTrackIDs(self):
		result = []
		item = self.firstChild()
		while item:
			if item.isSelected():
				result.append(item.trackID())
			item = item.itemBelow()
		return result

	def selectedTracks(self):
		result = []
		item = self.firstChild()
		while item:
			if item.isSelected():
				result.append(item.track())
			item = item.itemBelow()
		return result
	
	def allTracks(self):
		result = []
		item = self.firstChild()
		while item:
			result.append(item.track())
			item = item.itemBelow()
		return result

	def selectAll(self,select):
		item = self.firstChild()
		while item:
			if item.isSelected()!=select:
				self.setSelected(item,select)
			item = item.itemBelow()
		#self.repaint()

	def dragObject(self):
		selectedTracks = self.selectedTrackIDs()
		drag = LDellTrackDrag(selectedTracks,self)
		drag.dragCopy()
	
	def canImport(self):
		return False

	def canExport(self):
		return False
	
	def canBurn(self):
		return False

	def doSelectionChanged(self):
		self.emit(PYSIGNAL("selectionChanged"),(None,None))
		if self.anyItemsSelected():
			#LMusicPlayer.selectedTrack()
			pass

	def setSorting(self,column,ascending):
		KListView.setSorting(self,column,ascending)
		self.emitCurrentEnabledSortedTracks()

	def canPlay(self):
		return False
	
	def canGetInfo(self):
		return False
	
	def play(self):
		item = self.firstSelectedItem()
		if item:
			#print "playing selected item"
			item.track().play(self.playlist)
		else:
			#print "playing some item"
			LMusicPlayer.playItemFrom(self.library,self.playlist)

	def clear(self):
		item = self.firstChild()
		while item:
			item.die()
			item = item.itemBelow()
		KListView.clear(self)

	def setSorting(self,column,ascending):
		KListView.setSorting(self,column,ascending)
		self.settings['DDJ Library Sort Column'] = column
		self.settings['DDJ Library Sort Ascending'] = not not ascending

	def columnIndexChanged(self,section,fromIndex,toIndex):
		self.saveHeaderOrdering('DDJ Library Header Map')
	
	def columnSizeChanged(self,section,oldSize,newSize):
		self.saveHeaderSizes('DDJ Library Header Widths')

	def setSearchType(self,searchType):
		self.searchType = searchType
		self.loadTrackItems()

	def setSearchText(self,searchText):
		self.searchText = searchText
		self.loadTrackItems()
