#
# "@(#) $Id: LMusicRipper.py,v 1.14 2004/08/18 20:19:51 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from kdeemul import *
import os, time, re
from utils import *
from LsongsPaths import *
from LSettings import *
from LMusicPlayer import *
from LsongsProcess import *
import crushToAscii

#
# nuke any slashes that might confuse the OS
#
def cleanup(s):
	return re.sub(r"/",r"-",s)

class LRipperProcess(LsongsProcess):
	def __init__(self,track):
		#print "ripping",track.title
		self.track = track
		tmpPath = os.environ.get('TMP','/tmp')
		title = crushToAscii.crush(self.track.title)
		title = cleanup(title)
		self.wavPath = os.path.join(tmpPath,title+'.wav')
		LsongsProcess.__init__(self)
		self.CDDA_BLOCKSIZE = 2352

	def buildCommand(self):
		self.setExecutable("nice")
		self.setArguments(["cdparanoia","-e",str(self.track.trackNum),self.wavPath])
		self.emitEvent({'Source':'Ripper','Status':'BeginTrack','Track':self.track})
	
	def processLines(self,lines):
		line = lines[-1]
		tmp = line.split('@ ')
		if len(tmp)>1 and tmp[0]=='##: -2 [wrote] ':
			value = float(tmp[1])/self.CDDA_BLOCKSIZE*2
			currentTime = long(value*1000/75) - self.track.startTime
			self.maybeEmitEvent({'Source':'Ripper','Status':'Ripping','Track':self.track,'currentTime':currentTime,'totalTime':self.track.totalTime})
	
	def endProcess(self,process):
		self.emitEvent({'Source':'Ripper','Status':'EndTrack','Track':self.track,'filePath':self.wavPath})
		LsongsProcess.endProcess(self,process)

	def abortProcess(self,process):
		try: os.remove(self.wavPath)
		except: pass
		self.emitEvent({'Source':'Ripper','Status':'EndTrack','Track':self.track,'Interrupted':True})
		LsongsProcess.abortProcess(self,process)

class LMP3EncoderProcess(LsongsProcess):
	def __init__(self,track):
		#print "MP3 encoding",track.title
		self.track = track
		self.settings = LSettings.settings()
		tmpPath = os.environ.get('TMP','/tmp')
		title = crushToAscii.crush(self.track.title)
		title = cleanup(title)
		self.wavPath = os.path.join(tmpPath,title+'.wav')
		self.mp3Path = os.path.join(tmpPath,title+'.mp3')
		self.pattern = re.compile(r"\|(\d+)\|(\d+)\|")
		LsongsProcess.__init__(self)
	
	def buildCommand(self):
		bitRate = self.settings.get('Encoder Bit Rate',128000)/1000
		self.setExecutable("nice")
		args = [script("encode")]
		if not self.settings.get('Encoder VBR',False):
			args.append("-c")
		args.extend(["-b",str(bitRate),self.wavPath,self.mp3Path])
		self.setArguments(args)
		self.emitEvent({'Source':'Encoder','Status':'BeginTrack','Track':self.track})

	def processLines(self,lines):
		line = lines[-1]
		mo = self.pattern.match(line)
		if mo:
			currentTime = long(mo.group(1))
			totalTime = long(mo.group(2))
			self.maybeEmitEvent({'Source':'Encoder','Status':'Encoding','currentTime':currentTime,'totalTime':totalTime,'Track':self.track})

	def endProcess(self,process):
		try: os.remove(self.wavPath)
		except: pass
		#print "convert: processing tags for",self.mp3Path
		self.track.copyTagsInto(self.mp3Path)
		self.emitEvent({'Source':'Encoder','Status':'EndTrack','Track':self.track,'FilePath':self.mp3Path})
		LsongsProcess.endProcess(self,process)

	def abortProcess(self,process):
		try: os.remove(self.wavPath)
		except: pass
		try: os.remove(self.mp3Path)
		except: pass
		self.emitEvent({'Source':'Encoder','Status':'EndTrack','Track':self.track,'Interrupted':True})
		LsongsProcess.abortProcess(self,process)

class LOggEncoderProcess(LsongsProcess):
	def __init__(self,track):
		#print "Ogg encoding",track.title
		self.track = track
		self.settings = LSettings.settings()
		tmpPath = os.environ.get('TMP','/tmp')
		title = crushToAscii.crush(self.track.title)
		title = cleanup(title)
		self.wavPath = os.path.join(tmpPath,title+'.wav')
		self.oggPath = os.path.join(tmpPath,title+'.ogg')
		self.pattern = re.compile(r"\s*?\[\s*?(\d+)\.")
		LsongsProcess.__init__(self)
	
	def buildCommand(self):
		bitRate = self.settings.get('Encoder Bit Rate',128000)/1000
		self.setExecutable("nice")
		args = ["oggenc",self.wavPath,"-b",str(bitRate),"-o",self.oggPath]
		try:
			if self.track.title: args.extend(["-t",unikode(self.track.title).encode('latin-1')])
		except: pass
		try:
			if self.track.artist: args.extend(["-a",unikode(self.track.artist).encode('latin-1')])
		except: pass
		try:
			if self.track.album: args.extend(["-l",unikode(self.track.album).encode('latin-1')])
		except: pass
		try:
			if self.track.genre: args.extend(["-G",self.track.genre])
		except: pass
		try:
			if self.track.trackNum: args.extend(["-N",str(self.track.trackNum)])
		except: pass
		try:
			if self.track.trackCount: args.extend(["-c",("TRACKCOUNT=%d" % self.track.trackCount)])
		except: pass
		try:
			if self.track.year: args.extend(["-d",str(self.track.year)])
		except: pass
		self.convertLineEndings = True
		self.setArguments(args)
		self.emitEvent({'Source':'Encoder','Status':'BeginTrack','Track':self.track})

	def processLines(self,lines):
		line = lines[-1]
		mo = self.pattern.match(line)
		if mo:
			currentTime = long(mo.group(1))
			totalTime = 100
			self.maybeEmitEvent({'Source':'Encoder','Status':'Encoding','currentTime':currentTime,'totalTime':totalTime,'Track':self.track})

	def endProcess(self,process):
		try: os.remove(self.wavPath)
		except: pass
		self.emitEvent({'Source':'Encoder','Status':'EndTrack','Track':self.track,'FilePath':self.oggPath})
		LsongsProcess.endProcess(self,process)

	def abortProcess(self,process):
		try: os.remove(self.wavPath)
		except: pass
		try: os.remove(self.oggPath)
		except: pass
		self.emitEvent({'Source':'Encoder','Status':'EndTrack','Track':self.track,'Interrupted':True})
		LsongsProcess.abortProcess(self,process)

class LWavEncoderProcess(LsongsProcess):
	def __init__(self,track):
		self.track = track
		tmpPath = os.environ.get('TMP','/tmp')
		title = crushToAscii.crush(self.track.title)
		title = cleanup(title)
		self.wavPath = os.path.join(tmpPath,title+'.wav')
		LsongsProcess.__init__(self)
	
	def buildCommand(self):
		self.setExecutable('date')
		self.emitEvent({'Source':'Encoder','Status':'BeginTrack','Track':self.track,'FilePath':self.wavPath})
	
	def endProcess(self,process):
		self.emitEvent({'Source':'Encoder','Status':'EndTrack','Track':self.track,'FilePath':self.wavPath})
		LsongsProcess.endProcess(self,process)

	def abortProcess(self,process):
		try: os.remove(wavPath)
		except: pass
		self.emitEvent({'Source':'Encoder','Status':'EndTrack','Track':self.track,'Interrupted':True})
		LsongsProcess.abortProcess(self,process)
	
class LMusicRipper(QObject):
	def __init__(self):
		QObject.__init__(self)
		self.tracksToRip = []
		self.tracksToEncode = []
		self.ripProcess = None
		self.encodeProcess = None

	def ripNextTrack(self):
		if (self.ripProcess==None or not self.ripProcess.isRunning()):
			if len(self.tracksToRip)>0:
				self.gotStatus(None,{'Source':'Ripper','Status':'Begin'})
				track = self.tracksToRip[0]
				self.tracksToRip = self.tracksToRip[1:]
				self.ripProcess = LRipperProcess(track)
				QObject.connect(self.ripProcess,PYSIGNAL("done"),self.endRipTrack)
				QObject.connect(self.ripProcess,PYSIGNAL("status"),self.gotStatus)
				self.ripProcess.run()
			else:
				self.gotStatus(None,{'Source':'Ripper','Status':'End'})
		else:
			print "ripping is busy!"

	def endRipTrack(self,process):
		#print "done ripping",process.track.title
		if not process.interrupted:
			self.tracksToEncode.append(process.track)
			self.encodeNextTrack()
			self.ripNextTrack()

	def encodeNextTrack(self):
		if (self.encodeProcess==None or not self.encodeProcess.isRunning()):
			if len(self.tracksToEncode)>0:
				self.gotStatus(None,{'Source':'Encoder','Status':'Begin'})
				track = self.tracksToEncode[0]
				self.tracksToEncode = self.tracksToEncode[1:]
				encoder = LSettings.settings().get('Encoder','mp3')
				if encoder=='mp3':
					self.encodeProcess = LMP3EncoderProcess(track)
				elif encoder=='ogg':
					self.encodeProcess = LOggEncoderProcess(track)
				elif encoder=='wav':
					self.encodeProcess = LWavEncoderProcess(track)
				QObject.connect(self.encodeProcess,PYSIGNAL("done"),self.endEncodeTrack)
				QObject.connect(self.encodeProcess,PYSIGNAL("status"),self.gotStatus)
				self.encodeProcess.run()
			else:
				self.gotStatus(None,{'Source':'Encoder','Status':'End'})
		else:
			print "converting is busy!"
			
	def endEncodeTrack(self,process):
		#print "done encoding",process.track.title
		if not process.interrupted:
			self.encodeNextTrack()

	def gotStatus(self,process,status):
		#print status
		if status.has_key('Track'):
			status['Track'].setRipStatus(status)
		self.emit(PYSIGNAL('status'),(status,None))
	
	def ripTrack(self,track):
		self.ripTracks([track])

	def ripTracks(self,tracks):
		#print "ripping ",tracks
		if len(tracks)>0:
			self.tracksToRip.extend(tracks)
			self.ripNextTrack()
	
	def isRipping(self):
		if len(self.tracksToRip)>0: return True
		if len(self.tracksToEncode)>0: return True
		if self.ripProcess and self.ripProcess.isRunning(): return True
		if self.encodeProcess and self.encodeProcess.isRunning(): return True
		return False

	def killRipper(self):
		self.tracksToRip = []
		tracksToReset = self.tracksToEncode
		self.tracksToEncode = []
		if self.ripProcess:
			self.ripProcess.kill()
			self.ripProcess = None
			self.gotStatus(None,{'Source':'Ripper','Status':'End'})
		if self.encodeProcess:
			self.encodeProcess.kill()
			self.encodeProcess = None
			self.gotStatus(None,{'Source':'Encoder','Status':'End'})
		for track in tracksToReset:
			track.setRipStatus({'Source':'Encoder','Status':'EndTrack','Interrupted':True,'Track':track})
			try:
				tmpPath = os.environ.get('TMP','/tmp')
				title = crushToAscii.crush(track.title)
				title = cleanup(title)
				wavPath = os.path.join(tmpPath,title+'.wav')
				os.remove(wavPath)
			except: pass

	def static_singleton():
		global _ripperSingleton
		return _ripperSingleton

	singleton = staticmethod(static_singleton)

	def static_killCurrentRipper():
		LMusicRipper.singleton().killRipper()

	killCurrentRipper = staticmethod(static_killCurrentRipper)

_ripperSingleton = LMusicRipper()
