#
# "@(#) $Id: LPrefsDialog.py,v 1.7 2004/07/21 17:54:18 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from kdeemul import *
from LSettings import *
from utils import *

class LPrefsDialog(KDialogBase):
	def __init__(self):
		KDialogBase.__init__(self,KDialogBase.Tabbed,i18n("Preferences"),KDialogBase.Ok+KDialogBase.Cancel,\
			KDialogBase.Ok,None,"preferences",True)
		self.settings = LSettings.settings()
		self.qualities = {128000:unikode(i18n("Good Quality (128 kbps)")),160000:unikode(i18n("High Quality (160 kbps)")),192000:unikode(i18n("Higher Quality (192 kbps)")),256000:unikode(i18n("Best Quality (256 kpbs)"))}
		self.qualitiesKeys = self.qualities.keys()
		self.qualitiesKeys.sort()
		#self.encoders = {'mp3':'MP3 Encoder','aiff':'AIFF Encoder','ogg':'Ogg Encoder','wav':'WAV Encoder'}
		self.encoders = {'mp3':unikode(i18n('MP3 Encoder')),'ogg':unikode(i18n('Ogg Encoder')),'wav':unikode(i18n('WAV Encoder'))}
		self.encodersKeys = self.encoders.keys()
		self.encodersKeys.sort()
		self.insertActions = {'show':unikode(i18n("Show Songs")),'play':unikode(i18n("Begin Playing")),'import':unikode(i18n("Import Songs")),'importAndEject':unikode(i18n("Import Songs and Eject"))}
		self.insertActionsKeys = self.insertActions.keys()
		self.insertActionsKeys.sort()
		self.burnDiscTypes = {'audio':unikode(i18n("Audio CD")),'mp3':unikode(i18n("MP3 CD")),'data':unikode(i18n("Data CD"))}
		self.burnDiscTypesKeys = self.burnDiscTypes.keys()
		self.burnDiscTypesKeys.sort()
		self.add()
		self.load()
		self.adjustSize()
		self.setMinimumSize(QSize(200,200))

	def add(self):
		self.addGeneralItems()
		self.addImportingItems()
		self.addCDItems()
		self.addSharingItems()
		self.addStoreItems()
		self.addAdvancedItems()
	
	def addGeneralItems(self):
		return
		#page = self.addPage(i18n("General"))

	def addImportingItems(self):
		page = self.addPage(i18n("Importing"))
		hlayout = QHBoxLayout(page)
		hlayout.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))
		layout = QVBoxLayout()
		hlayout.addLayout(layout)
		hlayout.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))		
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))
		self.addImportOptionsItems(page,layout)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))

	def addImportOptionsItems(self,container,containerLayout):
		self.manageRadio = QRadioButton(i18n("Copy files on Import"),container,"manage")
		containerLayout.addWidget(self.manageRadio)
		self.unmanageRadio = QRadioButton(i18n("Use source files on Import"),container,"unmanage")
		containerLayout.addWidget(self.unmanageRadio)
		hlayout = QHBoxLayout()
		containerLayout.addLayout(hlayout)
		hlayout.addItem(QSpacerItem(10,1,QSizePolicy.Fixed,QSizePolicy.Fixed))
		self.modifyTagsCheckbox = QCheckBox(i18n("Allow tag modifications"),container,"modifiyTags")
		hlayout.addWidget(self.modifyTagsCheckbox)
		QObject.connect(self.manageRadio,SIGNAL("stateChanged(int)"),self.hitManaged)
		QObject.connect(self.unmanageRadio,SIGNAL("stateChanged(int)"),self.hitUnmanaged)
	
	def hitManaged(self,state):
		if state==QButton.On:
			self.unmanageRadio.setState(QButton.Off)
			self.modifyTagsCheckbox.setEnabled(False)
		else:
			self.unmanageRadio.setState(QButton.On)
	
	def hitUnmanaged(self,state):
		if state==QButton.On:
			self.manageRadio.setState(QButton.Off)
			self.modifyTagsCheckbox.setEnabled(True)
		else:
			self.manageRadio.setState(QButton.On)
		
	def addCDItems(self):
		page = self.addPage(i18n("CD"))
		# center horizontally
		hlayout = QHBoxLayout(page)
		hlayout.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))
		layout = QVBoxLayout()
		layout.setSpacing(5)
		hlayout.addLayout(layout)
		hlayout.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))
		
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))
		self.addCDInsertionItems(page,layout)
		self.addCDImportItems(page,layout)
		self.addCDBurnItems(page,layout)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))
	
	def addCDInsertionItems(self,container,containerLayout):
		layout = QHBoxLayout()
		containerLayout.addLayout(layout)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))
		
		label = QLabel(i18n("On CD Insert:"),container)
		label.setAlignment(Qt.AlignRight)
		layout.addWidget(label)
	
		self.onCDInsertItem = KComboBox(False,container)
		for action in self.insertActionsKeys:
			self.onCDInsertItem.insertItem(self.insertActions[action])
		layout.addWidget(self.onCDInsertItem)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))

	def addCDImportItems(self,container,containerLayout):
		layout = QGridLayout(4,2)
		containerLayout.addLayout(layout)
		layout.setSpacing(5)
		
		label = QLabel(i18n("Import Using:"),container)
		label.setAlignment(Qt.AlignRight)
		layout.addWidget(label,0,0)

		self.importUsingItem = KComboBox(False,container)
		for encoder in self.encodersKeys:
			self.importUsingItem.insertItem(self.encoders[encoder])
		layout.addWidget(self.importUsingItem,0,1)
		QObject.connect(self.importUsingItem,SIGNAL("highlighted(int)"),self.hitImportUsingItem)

		label = QLabel(i18n("Setting:"),container)
		label.setAlignment(Qt.AlignRight)
		layout.addWidget(label,1,0)

		self.importSettingsItem = KComboBox(False,container)
		for bitrate in self.qualitiesKeys:
			self.importSettingsItem.insertItem(self.qualities[bitrate])
		layout.addWidget(self.importSettingsItem,1,1)

		self.useVBRItem = QCheckBox(i18n("Variable Bit Rate"),container)
		layout.addWidget(self.useVBRItem,2,1)

		self.useErrorCorrectionItem = QCheckBox(i18n("Use error correction"),container)
		layout.addWidget(self.useErrorCorrectionItem,3,1)		
	
	def hitImportUsingItem(self,index):
		self.importSettingsItem.setEnabled(index!=2)
		self.useVBRItem.setEnabled(index==0)

	def addCDBurnItems(self,container,containerLayout):
		layout = QGridLayout(1,2)
		containerLayout.addLayout(layout)
		layout.setSpacing(3)

		label = QLabel(i18n("Burn Disc Format:"),container)
		label.setAlignment(Qt.AlignRight)
		layout.addWidget(label,0,0)
		
		self.burnDiscTypeItem = KComboBox(False,container)
		for action in self.burnDiscTypesKeys:
			self.burnDiscTypeItem.insertItem(self.burnDiscTypes[action])
		layout.addWidget(self.burnDiscTypeItem,0,1)
		#layout.addItem(QSpacerItem(0,1,QSizePolicy.Expanding,QSizePolicy.Fixed))

	def addSharingItems(self):
		pass
		#page = self.addPage(i18n("Sharing"))

	def addStoreItems(self):
		pass
		#page = self.addPage(i18n("Store"))

	def addAdvancedItems(self):
		pass
		#page = self.addPage(i18n("Advanced"))

	def addHDivider(self,container,containerLayout):
		containerLayout.addItem(QSpacerItem(1,2,QSizePolicy.Expanding,QSizePolicy.Fixed))
		w = QWidget(container)
		w.setPaletteBackgroundColor(QColor(0,0,0))
		w.setFixedHeight(1)
		w.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Fixed)
		containerLayout.addWidget(w)
		containerLayout.addItem(QSpacerItem(1,2,QSizePolicy.Expanding,QSizePolicy.Fixed))

	def load(self):
		self.loadGeneralItems()
		self.loadImportingItems()
		self.loadCDItems()
		self.loadSharingItems()
		self.loadStoreItems()
		self.loadAdvancedItems()
	
	def loadGeneralItems(self):
		pass
	
	def loadImportingItems(self):
		if self.settings.get('Managed',True):
			self.manageRadio.setOn(True)
		else:
			self.unmanageRadio.setOn(True)
		self.modifyTagsCheckbox.setOn(self.settings.get('Modify Tags',False))

	def loadCDItems(self):
		self.onCDInsertItem.setCurrentText(self.insertActions.get(self.settings.get("CD Insertion Action",'show'),"<Unknown>"))
		self.importUsingItem.setCurrentText(self.encoders.get(self.settings.get("Encoder","mp3"),"<Unknown>"))
		self.importSettingsItem.setCurrentText(self.qualities.get(self.settings.get("Encoder Bit Rate",128000),"<Unknown>"))
		self.useErrorCorrectionItem.setOn(self.settings.get("CD Error Correction",False))
		self.useVBRItem.setOn(self.settings.get("Encoder VBR",False))
		self.burnDiscTypeItem.setCurrentText(self.burnDiscTypes.get(self.settings.get("Burn Disc Type",'audio'),"<Unknown>"))

	def loadSharingItems(self):
		pass
	
	def loadStoreItems(self):
		pass
		
	def loadAdvancedItems(self):
		pass

	def unload(self):
		self.unloadGeneralItems()
		self.unloadImportingItems()
		self.unloadCDItems()
		self.unloadSharingItems()
		self.unloadStoreItems()
		self.unloadAdvancedItems()
		LSettings.singleton().writeXMLFile()
		
	def unloadGeneralItems(self):
		pass
	
	def unloadImportingItems(self):
		self.settings['Managed'] = self.manageRadio.isOn()
		self.settings['Modify Tags'] = self.modifyTagsCheckbox.isOn()
	
	def unloadCDItems(self):
		self.settings['CD Insertion Action'] = self.insertActionsKeys[self.onCDInsertItem.currentItem()]
		self.settings['Encoder'] = self.encodersKeys[self.importUsingItem.currentItem()]
		self.settings['Encoder Bit Rate'] = self.qualitiesKeys[self.importSettingsItem.currentItem()]
		self.settings['CD Error Correction'] = self.useErrorCorrectionItem.isOn()
		self.settings['Encoder VBR'] = self.useVBRItem.isOn()
		self.settings['Burn Disc Type'] = self.burnDiscTypesKeys[self.burnDiscTypeItem.currentItem()]

	def unloadSharingItems(self):
		pass
	
	def unloadStoreItems(self):
		pass
		
	def unloadAdvancedItems(self):
		pass

