#
# "@(#) $Id: LSourceView.py,v 1.16 2004/08/18 20:19:51 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from kdeemul import *
from Library import *
from LRadioLibrary import *
from LTrackDrag import *
from LDellTrackDrag import *
from LPlaylistDrag import *
from CDROM import *
from DellNJB import *
from DellLibrary import *
from LMusicCDMounter import *
from LMusicDellMounter import *
from LMusicDellExporter import *
from LMusicDellRemover import *
from LPlaylistWindow import *
from LsongsPaths import *

def removeDuplicates(items):
	result = {}
	for item in items:
		result[str(item)] = 1
	return result.keys()

def cmpRect(r1,r2):
	if r1.x()!=r2.x(): return False
	if r1.y()!=r2.y(): return False
	if r1.width()!=r2.width(): return False
	if r1.height()!=r2.height(): return False
	return True

	# To enforce sort, we precompute special keys
	# A is 'Library'
	# B is 'Stations'
	# C is 'Music Store'
	# D is 'Purchased Music'
	# E is CD/DVD devices
	# F+name is Smart Playlists
	# G+name is Normal Playlists
	# H+name is MP3 devices
	# I+name is Incoming Shared Playlists
	# J is Trash

class LSourceViewItem(QListViewItem):
	def __init__(self,container,name):
		QListViewItem.__init__(self,container,name)
		self.source = None
		self.name = name
		self.sortKey = QString(self.makeKey())
		self.wantsTrackDrags = False
		self.wantsDellTrackDrags = False
		self.wantsExternalTrackDrags = False
		self.wantsPlaylistDrags = False
		self.wantsFileDrags = False
		self.canBeRemoved = False
		self.hasContextMenu = True
		iconFileName = self.iconFileName()
		self.setDropEnabled(True)
		if iconFileName:
			iconPath = art(iconFileName)
			pixmap = QPixmap(iconPath)
			self.setPixmap(0,pixmap)
		self.canExport = False
		self.canBurn = False
		self.canEmpty = False
		self.library = Library.mainLibrary()
		self.libraryPlaylist = self.library.playlistWithName("Library")
		self.trash = self.library.playlistWithName("Trash")
		self.bgColor = None
		self.popup = None
	
	def contextMenu(self,pos,column,mainWindow):
		if self.popup==None:
			self.buildPopup(mainWindow)
		if self.popup!=None:
			self.loadPopup()
			self.popup.exec_loop(pos)

	def buildPopup(self,mainWindow):
		pass

	def loadPopup(self):
		pass

	def setBackgroundColor(self,color):
		self.bgColor = color
	
	def paintCell(self,p,cg,column,width,alignment):
		if self.bgColor:
			_cg = QColorGroup(cg)
			_cg.setColor(QColorGroup.Background,self.bgColor)
			_cg.setColor(QColorGroup.Base,self.bgColor)
			QListViewItem.paintCell(self,p,_cg,column,width,alignment)
		else:
			QListViewItem.paintCell(self,p,cg,column,width,alignment)

	def iconFileName(self):
		return None

	def makeKey(self):
		return '0'

	def key(self,column,ascending):
		return self.sortKey
		
	def okRename(self,column):
		oldName = self.text(0)
		QListViewItem.okRename(self,column)
		newName = unikode(self.text(0))
		if oldName!=newName:
			newName = newName.strip()
			if len(newName)==0:
				KMessageBox.error(None,i18n("Playlist names must not be empty"),i18n("Bad Playlist name"))
				self.setText(0,oldName)
				self.startRename(0)
			elif self.library.playlistWithName(newName)!=None:
					KMessageBox.error(None,i18n("Playlist names must be unique"),i18n("Duplicate Playlist name"))
					self.setText(0,oldName)
					self.startRename(0)
			else:
				self.changeName(newName)

	def changeName(self,newName):
		pass

	def doSourceChangeName(self,newName):
		self.source.name = newName

	def printDragFormats(self,event):
		print("formats:")
		n = 0
		while True:
			format = event.format(n)
			if (format==None):
				break
			print("  "+str(format))
			n = n+1
	
	def acceptDrag(self,event):
		return (self.wantsTrackDrags and LTrackDrag.canDecode(event)) or \
			(self.wantsDellTrackDrags and LDellTrackDrag.canDecode(event)) or \
			(self.wantsFileDrags and QUriDrag.canDecode(event)) or\
			(self.wantsPlaylistDrags and LPlaylistDrag.canDecode(event))
		# XXX DSM add external track drags

	def dropped(self,event):
		#print "dropped"
		#self.printDragFormats(event)
		trackIDs = LTrackDrag.decode(event)['TrackIDs']
		self.addTrackIDs(trackIDs)

	def addTrackIDs(self,trackIDs):
		pass
	
	def select(self,container):
		pass

	def doubleClick(self):
		#print "double clicked",self
		pass

#
# "Music store"
#
class LMusicStoreSourceViewItem(LSourceViewItem):
	def __init__(self,container):
		LSourceViewItem.__init__(self,container,i18n('Music Store'))
		self.hasContextMenu = False

	def makeKey(self):
		return 'C'

	def iconFileName(self): return "sourcemusicstore.png"

	def select(self,container):
		container.emit(PYSIGNAL("selectMusicStore"),(None,None))

#
# "Trash"
#
class LTrashSourceViewItem(LSourceViewItem):
	def __init__(self,container):
		LSourceViewItem.__init__(self,container,i18n('Trash'))
		self.wantsTrackDrags = True
		self.wantsPlaylistDrags = True
		self.wantsDellTrackDrags = True
		self.canEmpty = True

	def makeKey(self):
		return 'J'

	def iconFileName(self): return "sourcetrash.png"
	
	def select(self,container):
		container.emit(PYSIGNAL('selectTrash'),(None,None))
	
	def buildPopup(self,mainWindow):
		self.popup = QPopupMenu()
		self.emptyid = self.popup.insertItem(i18n("Empty &Trash"),mainWindow.slotEmptyTrash)
	
	def loadPopup(self):
		self.popup.setItemEnabled(self.emptyid,self.trash.hasTracks())

	#
	# handle drags to the trash
	#
	# if drag is playlist, remove the playlist
	# if drag is tracks:
	#    if source is playlist, remove tracks from playlist
	#    if source is library, remove tracks from all playlists and put in trash
	#    if source is DDJ, remove tracks from DDJ
	#
	def dropped(self,event):
		if LPlaylistDrag.canDecode(event):
			drag  = LPlaylistDrag.decode(event)
			playlist = self.library.playlistWithName(drag['Playlist'])
			if playlist:
				self.library.removePlaylist(playlist)
		elif LTrackDrag.canDecode(event):
			drag = LTrackDrag.decode(event)
			#print "trash got",drag
			playlist = self.library.playlistWithName(drag['Playlist'])
			trackIDs = drag['TrackIDs']
			if not playlist == self.trash:
				if playlist == self.libraryPlaylist:
					self.trash.addTrackIDs(trackIDs)
					self.library.removeTrackIDsFromAllPlaylists(trackIDs)
				else:
					playlist.removeTrackIDs(trackIDs)
		elif LDellTrackDrag.canDecode(event):
			drag = LDellTrackDrag.decode(event)
			trackIDs = drag['TrackIDs']
			LMusicDellRemover.removeTrackIDs(trackIDs)

#
# Shared playlist
#
class LSharedPlaylistSourceViewItem(LSourceViewItem):
	def __init__(self,container,name):
		LSourceViewItem.__init__(self,container,name)		
		self.hasContextMenu = False

	def makeKey(self):
		return 'I'+str(self.name)

	def iconFileName(self): return "sourcesharedmusic.png"

#
# "Stations" (streaming)
#
class LStationsSourceViewItem(LSourceViewItem):
	def __init__(self,container):
		LSourceViewItem.__init__(self,container,i18n("Stations"))

	def makeKey(self):
		return 'B'

	def iconFileName(self): return "sourceradio.png"
	
	def select(self,container):
		container.emit(PYSIGNAL('selectRadio'),(None,None))

##~ 	def buildPopup(self,mainWindow):
##~ 		self.popup = QPopupMenu()
##~ 		id = self.popup.insertItem(i18n("Add Station..."),self.addStation)
##~ 		id = self.popup.insertItem(i18n("Refresh Station List"),self.refresh)
		
	def addStation(self):
		LRadioLibrary.singleton().addStream()

	def refresh(self):
		mainWindow.slotRefresh()

#
# CD/DVD playlist
#
class LCDDVDSourceViewItem(LSourceViewItem):
	def __init__(self,container):
		name = CDLibrary.singleton().name
		LSourceViewItem.__init__(self,container,name)
		self.source = CDLibrary.singleton().playlistWithName('CD')

	def makeKey(self):
		return 'E'

	def iconFileName(self): return "sourcecddvd.png"

	def select(self,container):
		container.emit(PYSIGNAL("selectAudioCD"),(None,None))

	def buildPopup(self,mainWindow):
		self.popup = QPopupMenu()
		id = self.popup.insertItem(i18n("Import Tracks"),mainWindow.slotRIP)
		id = self.popup.insertItem(i18n("Eject"),mainWindow.slotEjectDisc)

#
# "Purchased Music"
#
class LPurchasedMusicSourceViewItem(LSourceViewItem):
	def __init__(self,container,source):
		LSourceViewItem.__init__(self,container,source['Name'])
		self.canBurn = True
		self.canExport = True
		self.hasContextMenu = False
	
	def makeKey(self):
		return 'D'

	def iconFileName(self): return "sourcepurchasedmusic.png"

#
# "Smart" playlist
#
class LSmartPlaylistSourceViewItem(LSourceViewItem):
	def __init__(self,container,source):
		LBurnExportSourceViewItem.__init__(self,container,source['Name'])
		self.source = source
		self.setRenameEnabled(0,True)
		self.canBurn = True
		self.canExport = True
		self.canBeRemoved = True

	def makeKey(self):
		return u'F%s' % unikode(self.name)

	def changeName(self,newName):
		self.doSourceChangeName(newName)

	def iconFileName(self): return "sourcesmartplaylist.png"

#
# Standard playlist
#
class LPlaylistSourceViewItem(LSourceViewItem):
	def __init__(self,container,source):
		LSourceViewItem.__init__(self,container,source.name)
		self.source = source
		self.setDragEnabled(True)
		self.setRenameEnabled(0,True)
		self.setDropEnabled(True)
		self.wantsTrackDrags = True
		self.wantsFileDrags = True
		self.canExport = True
		self.canBurn = True
		self.canBeRemoved = True

	def makeKey(self):
		return u'G%s' % unikode(self.name)

	def changeName(self,newName):
		self.doSourceChangeName(newName)

	def buildPopup(self,mainWindow):
		self.popup = QPopupMenu()
		id = self.popup.insertItem(i18n("Rename Playlist"),self.renamePlaylist)
		id = self.popup.insertItem(i18n("Delete Playlist"),self.deletePlaylist)
		self.exportId = self.popup.insertItem(i18n("Export Song List"),self.exportSongList)
		self.burnId = self.popup.insertItem(i18n("Burn Playlist to Disc"),self.burnPlaylistToDisc)

	def loadPopup(self):
		self.popup.setItemEnabled(self.burnId,CDR.canBurn())

	def exportSongList(self):
		self.source.exportSongList()
	
	def burnPlaylistToDisc(self):
		self.source.burnPlaylistToDisc()
	
	def renamePlaylist(self):
		self.startRename(0)

	def deletePlaylist(self):
		self.library.removePlaylist(self.source)
	
	def doubleClick(self):
		LPlaylistWindow.openOnPlaylist(self.source)

	#
	# allow file system drags
	# allow track drags from anywhere except self
	#
	def acceptDrag(self,event):
		if not self.wantsTrackDrags: return False
		elif self.wantsFileDrags and QUriDrag.canDecode(event): return True
		elif LTrackDrag.canDecode(event):
			drag = LTrackDrag.decode(event)
			playlist = self.library.playlistWithName(drag['Playlist'])
			if playlist!=self.source: return True
		elif LDellTrackDrag.canDecode(event):
			return True
		return False

	#
	# handle file system drags, add to both library and self
	# handle track drags
	# if from trash, add to library and remove from trash
	#
	def dropped(self,event):
		if LTrackDrag.canDecode(event):
			drag = LTrackDrag.decode(event)
			#print "playlist got",drag
			playlist = self.library.playlistWithName(drag['Playlist'])
			trackIDs = drag['TrackIDs']
			self.source.addTrackIDs(trackIDs)
			if playlist == self.trash:
				self.libraryPlaylist.addTrackIDs(trackIDs)
				self.trash.removeTrackIDs(trackIDs)
		elif LDellTrackDrag.canDecode(event):
			drag = LDellTrackDrag.decode(event)
			trackIDs = drag['TrackIDs']
			tracks = DellLibrary.singleton().tracksWithTrackIDs(trackIDs)
			#print "accepting Dell",tracks
			self.library.importDellFiles(tracks,self.source)
		elif QUriDrag.canDecode(event):
			uris = QStringList()
			if QUriDrag.decodeLocalFiles(event,uris):
				self.library.importFiles(removeDuplicates(uris),self.source)
		
	def iconFileName(self): return "sourceplaylist.png"

	def select(self,container):
		container.emit(PYSIGNAL("selectPlaylist"),(self.source,None))

#
# DDJ Player devices
#
class LDellDeviceSourceViewItem(LSourceViewItem):
	def __init__(self,container,name):
		LSourceViewItem.__init__(self,container,name)
		self.setDropEnabled(True)
		self.wantsFileDrags = True
		self.wantsTrackDrags = True
		self.canExport = True
		self.canBurn = True

	def makeKey(self):
		return 'H'+str(self.name)

	def buildPopup(self,mainWindow):
		self.popup = QPopupMenu()
		id = self.popup.insertItem(i18n("Sync"),mainWindow.slotSyncWithDevice)

	def iconFileName(self): return "sourcedevice.png"
	
	def select(self,container):
		container.emit(PYSIGNAL("selectDellPlayer"),(self,None))

	#
	# allow track drags from anywhere
	#
	def acceptDrag(self,event):
		if LTrackDrag.canDecode(event):
			return True
		return False

	#
	# handle track drags
	#
	def dropped(self,event):
		if LTrackDrag.canDecode(event):
			drag = LTrackDrag.decode(event)
			trackIDs = drag['TrackIDs']
			tracks = self.library.tracksWithTrackIDs(trackIDs)
			LMusicDellExporter.singleton().exportTracks(tracks)
		

#
# Master library playlist
#
class LLibrarySourceViewItem(LPlaylistSourceViewItem):
	def __init__(self,container,source):
		LPlaylistSourceViewItem.__init__(self,container,source)
		self.setRenameEnabled(0,False)
		self.wantsTrackDrags = True
		self.canExport = True
		self.setText(0,i18n("Media Library"))

	def makeKey(self):
		return 'A'

	def buildPopup(self,mainWindow):
		self.popup = QPopupMenu()
		id = self.popup.insertItem(i18n("Export Song List"),mainWindow.slotExportLibrary)

	def loadPopup(self):
		pass

	def iconFileName(self): return "sourcelibrary.png"

	def select(self,container):
		container.emit(PYSIGNAL("selectLibrary"),(None,None))

	#
	# allow file system drags
	# allow track drags from the trash only
	#
	def acceptDrag(self,event):
		if not self.wantsTrackDrags: return False
		elif self.wantsFileDrags and QUriDrag.canDecode(event): return True
		elif LTrackDrag.canDecode(event):
			drag = LTrackDrag.decode(event)
			playlist = self.library.playlistWithName(drag['Playlist'])
			if playlist==self.trash: return True
		elif LDellTrackDrag.canDecode(event):
			return True
		return False

	#
	# handle file system drags
	# and track drags if source is the trash
	#
	def dropped(self,event):
		#self.printDragFormats(event)
		if LTrackDrag.canDecode(event):
			drag = LTrackDrag.decode(event)
			#print "library got",drag
			playlist = self.library.playlistWithName(drag['Playlist'])
			trackIDs = drag['TrackIDs']
			if playlist == self.trash:
				self.libraryPlaylist.addTrackIDs(trackIDs)
				self.trash.removeTrackIDs(trackIDs)
		elif LDellTrackDrag.canDecode(event):
			drag = LDellTrackDrag.decode(event)
			trackIDs = drag['TrackIDs']
			tracks = DellLibrary.singleton().tracksWithTrackIDs(trackIDs)
			#print "library accepting Dell",tracks
			self.library.importDellFiles(tracks)
		elif QUriDrag.canDecode(event):
			uris = QStringList()
			if QUriDrag.decodeLocalFiles(event,uris):
				self.library.importFiles(removeDuplicates(uris),None)

	def doubleClick(self):
		pass

class LSourceView(QListView):
	def __init__(self,container,mainWindow):
		QListView.__init__(self,container,"sources")
		self.dellSource = None
		self._hasDDJ = False
		self.loadHeaders()
		self.loadSources()
		self.connect(self,SIGNAL("clicked(QListViewItem *)"),self.selectSourceItem)
		library = Library.mainLibrary()
		library.connect(PYSIGNAL("addedPlaylist"),self.addedPlaylist)
		library.connect(PYSIGNAL("removedPlaylist"),self.removedPlaylist)
		library.connect(PYSIGNAL("renamedPlaylist"),self.renamedPlaylist)
		# XXX DSM added next line to support KDE emulation
		QObject.connect(self,PYSIGNAL("dropped(QDropEvent *, QListViewItem *)"),self.droppedOnItem)
		# XXX DSM removed following line to support KDE emulation
		# QObject.connect(self,SIGNAL("dropped(QDropEvent *, QListViewItem *)"),self.droppedOnItem)
		QObject.connect(self,SIGNAL("doubleClicked(QListViewItem *)"),self.doDoubleClicked)
		QObject.connect(self,SIGNAL("contextMenuRequested(QListViewItem *, const QPoint &, int)"),self.contextMenu)
		self.setAcceptDrops(True)
		self.viewport().setAcceptDrops(True)
		#self.setDropVisualizer(False)
		#self.setDropHighlighter(True)
		self.setDefaultRenameAction(QListView.Accept)
		QObject.connect(LMusicCDMounter.singleton(),PYSIGNAL("cdInserted"),self.cdInserted)
		QObject.connect(CDROM.singleton(),PYSIGNAL("cdEjected"),self.cdEjected)
		QObject.connect(LMusicDellMounter.singleton(),PYSIGNAL("dellInserted"),self.dellInserted)
		QObject.connect(DellNJB.singleton(),PYSIGNAL("dellEjected"),self.dellEjected)
		self.mOldDropHighlighter = QRect(0,0,0,0)
		self.mainWindow = mainWindow
		
		self.oldCurrent = 0
		self.dropItem = 0
		self.autoOpenTime = 750
		self.openTimer = QTimer(self)
		self.flashTimer = QTimer(self)
		self.connect(self.openTimer,SIGNAL("timeout()"),self.openFolder)
		self.connect(self.flashTimer,SIGNAL("timeout()"),self.flashFunc)
		self.flashing = False
		self.flashCount = 0
		self.flashItem = None
		self.fullWidth = False
	
	def openFolder(self):
		self.openTimer.stop()
		if self.dropItem and not self.dropItem.isOpen():
			self.dropItem.setOpen(True)
			self.dropItem.repaint()
	
	def flashFunc(self):
		if self.flashCount == 6:
			self.flashTimer.stop()
			self.flashCount = 0
			return
		elif self.flashCount==0:
			self.flashTimer.start(50)
		self.flashCount = self.flashCount+1
		if self.flashing:
			self.flashing = False
			self.flashItem.setBackgroundColor(None)
			self.flashItem.repaint()
		else:
			self.flashing = True
			self.flashItem.setBackgroundColor(QColor(0,0,255))
			self.flashItem.repaint()
	
	def contentsDragEnterEvent(self,e):
		self.oldCurrent = self.currentItem()
		self.contentsDragMoveEvent(e)
	
	def contentsDragMoveEvent(self,e):
		vp = self.contentsToViewport(e.pos())
		i = self.itemAt(vp)
		if i:
			if i.acceptDrag(e):
				e.accept()
				if i!=self.dropItem:
					if self.dropItem:
						self.dropItem.setBackgroundColor(None)
						self.dropItem.repaint()
					i.setBackgroundColor(QColor(200,214,255))
					i.repaint()
					self.openTimer.stop()
					self.dropItem = i
					self.openTimer.start(self.autoOpenTime)
			else:
				if self.dropItem:
					self.dropItem.setBackgroundColor(None)
					self.dropItem.repaint()
					self.dropItem = 0
				e.ignore()
		else:
			if self.dropItem:
				self.dropItem.setBackgroundColor(None)
				self.dropItem.repaint()
			#e.accept()
			self.openTimer.stop()
			self.dropItem = 0
	
	def contentsDragLeaveEvent(self,e):
		self.openTimer.stop()
		if self.dropItem:
			self.dropItem.setBackgroundColor(None)
			self.dropItem.repaint()
			self.dropItem = 0
		self.setCurrentItem(self.oldCurrent)
		self.setSelected(self.oldCurrent,True)
	
	def contentsDropEvent(self,e):
		self.openTimer.stop()
		if self.dropItem:
			self.dropItem.setBackgroundColor(None)
			self.dropItem.repaint()
			self.dropItem = 0
		item = self.itemAt(self.contentsToViewport(e.pos()))
		if item and item.acceptDrag(e):
			if item.dropped(e):
				self.flashItem = item
				self.flashFunc()
		elif self.acceptDrag(e):
			self.dropped(e)
	
	def setFullWidth(self,bool = True):
		self._fullWidth = bool
		self.header().setStretchEnabled(self.fullWidth(),self.columns()-1)
	
	def fullWidth(self):
		return self._fullWidth

	def selectSourceItem(self,item):
		if item:
			item.select(self)
		#self.selectSource(str(item.text(0)))

	def selectSource(self,source):
		#print("hit source "+str(source))
		self.emit(PYSIGNAL("selectSource"),(source,None))

	def loadHeaders(self):
		self.addColumn(i18n("Source"))
		self.setFullWidth(True)
		self.setSelectionMode(QListView.Single)

	def loadSources(self):
		self.cdSource = None
		self.dellSource = None
		self.clear()
		lib = Library.mainLibrary()
		playlists = lib.playlists
		for playlist in playlists:
			if playlist.master:
				self.master = LLibrarySourceViewItem(self,playlist)
			elif playlist.trash:
				LTrashSourceViewItem(self)
			elif playlist.purchased:
				LPurchasedMusicSourceViewItem(self,playlist)
			elif playlist.smart:
				LSmartPlaylistSourceViewItem(self,playlist)
			else:
				LPlaylistSourceViewItem(self,playlist)
		LStationsSourceViewItem(self)
		#LSharedPlaylistSourceViewItem(self,"Joe's Shared Music")
		#LDeviceSourceViewItem(self,"Jane's Player")
		LMusicStoreSourceViewItem(self)
	
	def loadRemoveables(self):
		if self.hasCD():
			self.cdInserted()
		if self.hasDell():
			self.dellInserted()

	def addedPlaylist(self,library,playlist):
		self.loadSources()
		self.loadRemoveables()
		item = self.findItem(playlist.name,0)
		if item:
			item.startRename(0)
		
	def removedPlaylist(self,library,playlist):
		#print "removing",playlist.name
		item = self.findItem(playlist.name,0)
		if item:
			if item.isSelected():
				self.setSelected(self.master,True)
				self.master.select(self)
			self.takeItem(item)
			del item

	def renamedPlaylist(self,oldName,newName):
		self.loadSources()
		self.loadRemovables()

	def droppedOnItem(self,event,item):
		pos = self.contentsToViewport(event.pos())
		saveSelectedItem = self.selectedItem()
		item = self.itemAt(pos)
		if item:
			item.dropped(event)
		# XXX DSM workaround for Qt bug?
		self.repaint()
		if saveSelectedItem:
			try: # it might have been destroyed!
				#print "attempting refresh of",saveSelectedItem.text(0)
				self.setSelected(saveSelectedItem,False)
				self.setSelected(saveSelectedItem,True)
			except: pass

	#
	# handle the creation of playlists by drags into empty areas
	#
	def dropped(self,event):
		if LTrackDrag.canDecode(event):
			#print "trackIDs dropped nowhere"
			drag = LTrackDrag.decode(event)
			trackIDs = drag['TrackIDs']
			library = Library.mainLibrary()
			library.addPlaylistFromTrackIDs(trackIDs)
		# XXX DSM handle file drags here?

	def acceptDrag(self,event):
		if LTrackDrag.canDecode(event): return True
		# XXX DSM handle file drags here?
		return False

	def dragObject(self):
		selectedItem = self.selectedItem()
		drag = LPlaylistDrag(selectedItem.source,self)
		drag.dragCopy()

	def doDoubleClicked(self,item):
		item.doubleClick()

	def contextMenu(self,item,pos,column):
		if item!=None and item.hasContextMenu:
			item.contextMenu(pos,column,self.mainWindow)
		#self.doDoubleClicked(item)
		#self.popup.exec_loop(pos)

	def hasCD(self):
		return CDROM.singleton().hasCD()
		
	def cdInserted(self):
		if not self.cdSource:
			self.cdSource = LCDDVDSourceViewItem(self)

	def cdEjected(self):
		if self.cdSource:
			if self.cdSource.isSelected:
				self.setSelected(self.master,True)
				self.master.select(self)
			self.takeItem(self.cdSource)
			del self.cdSource
			self.cdSource = None

	def hasDell(self):
		return self._hasDDJ

	def dellInserted(self):
		self._hasDDJ = True
		if self.dellSource==None:
			self.dellSource = LDellDeviceSourceViewItem(self,"Dell Digital Jukebox")

	def dellEjected(self):
		self._hasDDJ = False
		if self.dellSource:
			if self.dellSource.isSelected():
				self.setSelected(self.master,True)
				self.master.select(self)
			self.takeItem(self.dellSource)
			del self.dellSource
			self.dellSource = None
	
	def showSongSource(self,track,library,playlist):
		item = self.firstChild()
		while item:
			if item.source == playlist:
				self.setSelected(item,True)
				item.select(self)
				return True
			item = item.itemBelow()
		return False

	#
	# The following three routines are a workaround to a bug
	# in KDE 3.1 when highlighting items that are a drop target
	# They attempt to overload the drop visualizer position to
	# also indicate the drop highlight position. Unfortunately,
	# these should be different items. The main reason we
	# have to override so much is that the highlight rectangle
	# is stored in some private structures within the KListView
	# instance
	#
	# This is a not a complete fix - I have made no attempt to
	# preserve the behavior of the drop visualizer since I don't
	# need it.
	#
##~ 	def contentsDragMoveEvent(self,event):
##~ 		pos = self.contentsToViewport(event.pos())
##~ 		if self.acceptDrag(event):
##~ 			event.acceptAction()
##~ 			if self.dropHighlighter():
##~ 				item = self.itemAt(pos)
##~ 				tmpRect = self.drawItemHighlighter(None, item)
##~ 				if not cmpRect(tmpRect,self.mOldDropHighlighter):
##~ 					self.cleanItemHighlighter()
##~ 					self.mOldDropHighlighter=tmpRect
##~ 					self.viewport().repaint(tmpRect)
##~ 		else:
##~ 			self.cleanItemHighlighter()
##~ 			event.ignore()
##~ 
##~ 	def cleanItemHighlighter(self):
##~ 		if self.mOldDropHighlighter.isValid():
##~ 			rect = self.mOldDropHighlighter
##~ 			self.mOldDropHighlighter = QRect()
##~ 			self.viewport().repaint(rect, True)
##~ 
##~ 	def viewportPaintEvent(self,e):
##~ 		QListView.viewportPaintEvent(self,e)
##~ 		if self.mOldDropHighlighter.isValid() and e.rect().intersects(self.mOldDropHighlighter):
##~ 			painter = QPainter(self.viewport())
##~ 			self.style().drawPrimitive(QStyle.PE_FocusRect, painter, self.mOldDropHighlighter, self.colorGroup(),QStyle.Style_FocusAtBorder)
