#
# "@(#) $Id: Library.py,v 1.20 2004/08/04 01:01:47 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from kdeemul import *
from MP3Track import *
from OggTrack import *
from WavTrack import *
from SunAUTrack import *
from AIFFTrack import *
from RealAudioTrack import *
from WindowsMediaTrack import *
from Playlist import *
from PListParser import *
import os, errno, shutil, fnmatch, time
from LMusicImporter import *
from LMusicDellImporter import *
import crushToAscii
import cPickle
from utils import *
_mainLibrary = None

def normalizeName(s):
	s = re.sub(r":",r"-",s)
	s = re.sub(r"/",r"-",s)
	s = re.sub(r"\\",r"-",s)
	return s

#
# return a flattened list of files that match the given pattern
# filter out any trashed files or Apple resource forks
#
def listFiles(root,patterns = '*',recurse=1,return_folders=0):
	pattern_list  = patterns.split(';')
	class Bunch:
		def __init__(self,**kwds): self.__dict__.update(kwds)
	arg = Bunch(recurse=recurse,pattern_list=pattern_list, return_folders=return_folders,results=[])
	
	def visit(arg,dirname,files):
		basename = os.path.basename(dirname)
		if basename=='.AppleDouble' or basename=='Trash':
			files[:] = []
			return
		for name in files:
			fullname = os.path.normpath(os.path.join(dirname,name))
			if arg.return_folders or os.path.isfile(fullname):
				for pattern in arg.pattern_list:
					if fnmatch.fnmatch(name,pattern):
						arg.results.append(fullname)
						break
		if not arg.recurse: files[:] = []
	os.path.walk(root,visit,arg)
	return arg.results

def removeDuplicates(items):
	result = {}
	for item in items:
		result[item] = 1
	return result.keys()

class Library(object):
	def __init__(self):
		self._qObject = QObject()
		self._tracks = {}
		self._playlists = []

	def qObject(self):
		return self._qObject

	def emit(self,signal,args):
		self._qObject.emit(signal,args)

	def connect(self,signal,target):
		QObject.connect(self._qObject,signal,target)

	def clearTracks(self):
		self._tracks = {}

	def getPlaylists(self):
		playlists = []
		for playlist in self._playlists:
			playlists.append(playlist)
		return playlists

	playlists = property(getPlaylists)

	def uniquePlaylistID(self):
		newID = 100
		for playlist in self._playlists:
			playlistID = playlist.playlistID
			if playlistID>=newID:
				newID = playlistID+1
		return newID

	def uniquePlaylistName(self):
		return self.uniquePlaylistNameFrom(str(i18n("untitled playlist")))
	
	def uniquePlaylistNameFrom(self,baseName):
		if self.playlistWithName(baseName)==None:
			return baseName
		index = 1
		while True:
			name = baseName+" "+str(index)
			if self.playlistWithName(name)==None:
				return name
			index = index+1

	def uniquePlaylistNameFromTracks(self,tracks):
		tags = None
		for track in tracks:
			tags = track.loadTagsInto(tags)
		name = None
		if tags!=None:
			if tags.has_key('title'):
				name = self.uniquePlaylistNameFrom(tags['title'])
			elif tags.has_key('artist'):
				if tags.has_key('album'):
					name = self.uniquePlaylistNameFrom("%s - %s" % (tags['artist'],tags['album']))
				else:
					name = self.uniquePlaylistNameFrom(tags['artist'])
			elif tags.has_key('album'):
				name = self.uniquePlaylistNameFrom(tags['album'])
		return name

	def addPlaylistFromTrackIDs(self,trackIDs):
		tracks = self.tracksWithTrackIDs(trackIDs)
		name = self.uniquePlaylistNameFromTracks(tracks)
		playlist = self.addNewPlaylist(name)
		playlist.addTrackIDs(trackIDs)		
		return playlist

	def playlistWithName(self,name):
		for playlist in self._playlists:
			#print type(playlist.name),type(name)
			if unikode(playlist.name)==unikode(name):
				return playlist
		return None

	def addNewPlaylist(self,name = None,master=False,trash=False):
		#print "adding playlist",name
		playlist = Playlist(self)
		if name!=None:
			playlist.name = name
		if master:
			playlist.master = True
		if trash:
			playlist.trash = True
		self._playlists.append(playlist)
		self.emit(PYSIGNAL('addedPlaylist'),(self,playlist))
		return playlist

	def removePlaylist(self,playlist):
		self._playlists.remove(playlist)
		self.emit(PYSIGNAL('removedPlaylist'),(self,playlist))
	
	def uniqueTrackID(self):
		newID = 1
		for trackKey in self._tracks.keys():
			track = self._tracks[trackKey]
			trackID = track.trackID
			if trackID>=newID:
				newID = trackID+1
		return newID

	def trackWithTrackID(self,trackID):
		try: return self._tracks[trackID]
		except: None

	def tracksWithTrackIDs(self,trackIDs):
		result = []
		for trackID in trackIDs:
			result.append(self.trackWithTrackID(trackID))
		return result

	def addTrack(self,track):
		self._tracks[track.trackID] = track
		#print "adding",track
		self.emit(PYSIGNAL('addedTrack'),(self,track))

	def removeTrackIDsFromAllPlaylists(self,trackIDs):
		for playlist in self._playlists:
			if not playlist.trash:
				playlist.removeTrackIDs(trackIDs)

	def tracks(self):
		return self._tracks
	
	def changedTrackAttribute(self,track,name,oldValue,newValue):
		if name=='artist' or name=='album':
			self.emit(PYSIGNAL("changedTrackArtistOrAlbum"),(self,track))

	def getPList(self):
		pList = {'Major Version':1,'Minor Version':0,'Application Version':'1.0'}
		tracks = {}
		for trackKey in self._tracks.keys():
			track = self._tracks[trackKey]
			tracks[str(track.trackID)] = track.pList
		pList['Tracks'] = tracks
		playlists = []
		for playlist in self._playlists:
			playlists.append(playlist.pList)
		pList['Playlists'] = playlists
		return pList

	def setPList(self,pList):
		#print "parsing tracks"
		tracks = pList['Tracks']
		progress = QProgressDialog("Reading",None,len(tracks),None,"progress",True)
		progress.setCaption(i18n("Building library"))
		progress.setTotalSteps(len(tracks))
		then = long(time.time()*4)
		index = 0
		for trackKey in tracks.keys():
			now = long(time.time()*4)
			updateProgress = False
			if now!=then:
				progress.setProgress(index)
				updateProgress = True
				then = now
			# XXX DSM handle different media types here
			trackPList = tracks[trackKey]
			kind = trackPList['Kind']
			if kind=='MPEG audio file':
				track = MP3Track(self)
			elif kind=='Ogg audio file':
				track = OggTrack(self)
			elif kind=='WAV audio file':
				track = WavTrack(self)
			elif kind=='Sun AU audio file':
				track = SunAUTrack(self)
			elif kind=='AIFF audio file' or kind=='AIFC audio file':
				track = AIFFTrack(self)
			elif kind=='Real Audio audio file':
				track = RealAudioTrack(self)
			elif kind=='Windows Media audio file':
				track = WindowsMediaTrack(self)
			else:
				print "Can't find track of kind",kind
				track = None
			if track!=None:
				track.pList = trackPList
				if updateProgress: progress.setLabelText(track.title)
				self._tracks[int(track.trackID)] = track
			index = index+1
		#print "parsing playlists"
		playlists = pList['Playlists']
		progress.setCaption(i18n("Building playlists"))
		progress.setTotalSteps(len(playlists))
		index = 0
		for playlistPList in playlists:
			progress.setProgress(index)
			# XXX DSM handle smart playlists here
			playlist = Playlist(self)
			playlist.pList = playlistPList
			progress.setLabelText(playlist.name)
			self._playlists.append(playlist)
			index = index+1
		progress.setProgress(len(playlists))

	pList = property(getPList,setPList)
	
	def writeXMLFile(self):
		pass

	def getMainLibrary():
		global _mainLibrary
		if _mainLibrary==None:
			_mainLibrary = MainLibrary()
			if not _mainLibrary.readBinaryFile():
				if not _mainLibrary.readXMLFile():
					_mainLibrary.addNewPlaylist("Library",master=True)
					_mainLibrary.addNewPlaylist("Trash",trash=True)
		return _mainLibrary

	mainLibrary = staticmethod(getMainLibrary)

class FileLibrary(Library):
	def __init__(self):
		Library.__init__(self)
		self.createExtensions()
		self.ensureDirectories()
	
	def createExtensions(self):
		self.validExtensions = self.filter().split(';')
	
	def filter(self):
		return "*.mp3"  # patterns separated by semicolons
	
	def fileFilter(self):
		return "*.mp3|Music Files"  # patterns separated by spaces
	
	def ensureDirectories(self):
		pass

	def musicPath(self):
		return "/tmp"

	def isValidFile(self,filePath):
		if os.path.exists(filePath):
			if os.path.isdir(filePath):
				files = listFiles(filePath,self.filter())
				for file in files:
					if self.isValidFile(file):
						return True
				return False
			else:
				filePath = filePath.lower()
				for ext in self.validExtensions:
					if fnmatch.fnmatch(filePath,ext):
						return True
		else:
			print "FileLibrary::isValidFile: path doesn't exist:",path
		return False

	def canImportFiles(self,uris):
		for uri in uris:
			if self.isValidFile(uri):
				return True
		return False

	def removeExisting(self,files):
		result = []
		for file in files:
			found = False
			for trackKey in self._tracks.keys():
				track = self._tracks[trackKey]
				if unikode(track.location)==unikode(file) or unikode(track.originalLocation)==unikode(file):
					#print file,"already in Library"
					found = True
					break
			if not found:
				result.append(file)
		return result

	def ensureDirectories(self):
		self.ensureDirectory(self.musicPath())

	def ensureDirectory(self,path):
		try:
			os.makedirs(path,0777)
		except OSError, err:
			if err.errno != errno.EEXIST or not os.path.isdir(path):
				raise

	def moveFile(self,fileName,fromPath,toPath):
		#print "moving "+fileName+" from "+fromPath+" to "+toPath
		self.ensureDirectory(toPath)
		newFileName = self.uniqueName(fileName,toPath)
		shutil.move(os.path.join(fromPath,fileName),os.path.join(toPath,newFileName))
		return newFileName

	def copyFile(self,fileName,fromPath,toPath):
		#print "copying "+fileName+" from "+fromPath+" to "+toPath
		self.ensureDirectory(toPath)
		newFileName = self.uniqueName(fileName,toPath)
		shutil.copy(os.path.join(fromPath,fileName),os.path.join(toPath,newFileName))
		return newFileName

	def removeIfEmpty(self,path):
		if os.path.exists(path):
			if len(os.listdir(path))==0:
				os.rmdir(path)
				(path,blah) = os.path.split(path)
				if len(os.listdir(path))==0:
					os.rmdir(path)
				return True
		return False

	def makePathFor(self,artist,album):
		if album==None or len(album)==0:
			album = "Unknown Album"
		if artist==None or len(artist)==0:
			artist = "Unknown Artist"
		artist = normalizeName(crushToAscii.crush(artist))
		album = normalizeName(crushToAscii.crush(album))
		return os.path.join(self.musicPath(),str(artist),str(album))

	def uniqueName(self,fileName,directory):
		altName = crushToAscii.crush(fileName)
		if os.path.exists(os.path.join(directory,altName)):
			(root,ext) = os.path.splitext(fileName)
			index = 1
			while True:
				altName = root+" "+str(index)+ext
				if not os.path.exists(os.path.join(directory,altName)):
					break
				index = index+1
		return altName

	def killTrackIDs(self,trackIDs):
		for trackID in trackIDs:
			track = self.trackWithTrackID(trackID)
			if track:
				track.kill()
			del self._tracks[trackID]

class MainLibrary(FileLibrary):
	def __init__(self):
		FileLibrary.__init__(self)

	def filter(self):
		return "*.mp3;*.ogg;*.wav;.au;.aiff;.aifc;*.ra;*.wmx;*.wma"

	def fileFilter(self):
		return "*.mp3 *.ogg *.wav *.au *.aiff *.aifc *.ra *.wmx *.wma|Music Files"

	def libraryPath(self):
		return os.path.join(os.path.expanduser('~'),'My Documents','My Music','Lsongs')

	def xmlPath(self):
		return os.path.join(self.libraryPath(),'LsongsMusicLibrary.xml')

	def binaryPath(self):
		return os.path.join(self.libraryPath(),'LsongsMusicLibrary.bin')
		
	def musicPath(self):
		return os.path.join(self.libraryPath(),'LsongsMusic')

	#
	# asynch import of Dell DJ files
	#
	def importDellFiles(self,tracks,playlist = None):
		LMusicDellImporter.singleton().importTracks(tracks,playlist)

	#
	# asynchronous import of multiple files
	#
	def importFiles(self,uris,playlist = None):
		#print "importing files",uris," to playlist",playlist
		files = []
		for uri in uris:
			if os.path.exists(uri):
				if os.path.isdir(uri):
					files.extend(listFiles(uri,self.filter()))
				else:
					files.append(uri)
		files = self.removeExisting(files)
		#print "importing",files
		if len(files)>0:
			files = removeDuplicates(files)
			files.sort()
			LMusicImporter.singleton().importFiles(files,self,playlist)
	
	def finishImportFile(self,track,playlist = None):
		masterList = self.playlistWithName("Library")
		track.addToLibrary(self)
		if masterList!=None:
			masterList.addTrack(track)
		if playlist!=None and playlist!=masterList:
			playlist.addTrack(track)
		#self.writeBinaryFile()

	#
	# simple synchronous import of a single file,
	# used by the CD ripper
	#
	def importFile(self,filePath,managed = False):
		# XXX DSM handle other rip file types here
		if type(filePath).__name__=='unicode':
			filePath = filePath.encode('latin-1')
		track = self.trackForFilePath(filePath)
		#print "importing",filePath,"into",track
		track.setFile(filePath)
		track.managed = True
		self.finishImportFile(track)
		return track

	def trackForFilePath(self,fullPath):
		(srcPath,fileName) = os.path.split(fullPath)
		(root,ext) = os.path.splitext(fileName)
		ext = ext.lower()
		track = None
		if ext=='.mp3':
			track = MP3Track()
		elif ext=='.ogg':
			track = OggTrack()
		elif ext=='.wav':
			track = WavTrack()
		elif ext=='.au':
			track = SunAUTrack()
		elif ext=='.aiff' or ext=='.aifc':
			track = AIFFTrack()
		elif ext=='.ra':
			track = RealAudioTrack()
		elif ext=='.wmx' or ext=='.wma':
			track = WindowsMediaTrack()
		return track

	def readXMLFile(self):
		#print "reading XML file"
		fileName = self.xmlPath()
		if os.path.exists(fileName):
			try:
				reader = PListReader()
				pList = reader.parse(fileName)
				self.pList = pList[0]
				self.sanityCheck()
			except: return False
			self.emit(PYSIGNAL("readLibrary"),(None,None))
			return True
		else:
			return False

	def readBinaryFile(self):
		#print "reading binary file"
		fileName = self.binaryPath()
		if os.path.exists(fileName):
			try:
				#print "about to read binary"
				fd = open(fileName,"rb")
				self.pList = cPickle.load(fd)
				self.sanityCheck()
				fd.close()
				#print "successfully read binary"
			except: return False
			self.emit(PYSIGNAL("readLibrary"),(None,None))
			return True
		else:
			return False

	#
	# check to make sure some basic things are working from the
	# loaded file - like there's a "Library" and a "Trash" and
	# that there are no "orphan" tracks
	#
	# there's a separate command to identify references to files
	# that no longer exist, or bring in files in the managed
	# directories that have no existing references
	#
	def sanityCheck(self):
		#
		# be sure there's a Library
		#
		mainlib = self.playlistWithName("Library")
		if mainlib==None:
			# Library missing, make one and add tracks
			print "missing Media Library"
			mainlib = self.addNewPlaylist("Library")
			mainlib.master = True
			for trackKey in self._tracks.keys():
				track = self._tracks[trackKey]
				mainlib.addTrack(track)
		#
		# be sure there's a Trash
		#
		trash = self.playlistWithName("Trash")
		if trash==None:
			print "missing Trash"
			# Trash missing, make an empty one
			trash = self.addNewPlaylist("Trash")
			trash.trash = True
		#
		# move any tracks not in the Library into
		# the Trash
		#
		for trackKey in self._tracks.keys():
			track = self._tracks[trackKey]
			if not mainlib.containsTrack(track):
				if not trash.containsTrack(track):
					trash.addTrack(track)
			
	def writeBinaryFile(self):
		fd = open(self.binaryPath(),"wb")
		try: cPickle.dump(self.getPList(),fd,cPickle.HIGHEST_PROTOCOL)
		except: cPickle.dump(self.getPList(),fd,True)
		fd.close()

	def writeXMLFile(self):
		writer = PListWriter()
		writer.unparse([self.pList],self.xmlPath())		
		
	def emptyTrash(self):
		trash = self.playlistWithName("Trash")
		trackIDs = trash.getTrackIDs()
		self.removeTrackIDsFromTrash(trackIDs)
	
	def removeTrackIDsFromTrash(self,trackIDs):
		trash = self.playlistWithName("Trash")
		trash.removeTrackIDs(trackIDs)
		self.killTrackIDs(trackIDs)
	
	def killTrackIDs(self,trackIDs):
		FileLibrary.killTrackIDs(self,trackIDs)
		self.writeBinaryFile()

if __name__=='__main__':
	l = MainLibrary()
	l.readXMLFile()
	print l.pList

