#
# "@(#) $Id: Playlist.py,v 1.13 2004/08/04 19:36:54 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from kdeemul import *
from PListParser import *
from LMusicBurner import *
from LBurnerSettingsDialog import *
import CDR
from utils import *

class Playlist(object):
	def __init__(self,library = None):
		self._qObject = QObject()
		self._library = library
		self._trackItems = []
		self.master = False
		self.trash = False
		self.purchased = False
		self.smart = False
		if library:
			self.name = library.uniquePlaylistName()
			self.playlistID = library.uniquePlaylistID()
		else:
			self.name = ""
			self.playlistID = 4242

	def qObject(self):
		return self._qObject
	
	def library(self):
		return self._library

	def emit(self,signal,args):
		self._qObject.emit(signal,args)

	def connect(self,signal,target):
		QObject.connect(self._qObject,signal,target)

	def getName(self):
		return self._name
	def setName(self,name):
		try:    oldName = self._name
		except: oldName = ""
		self._name = name
		self.emit(PYSIGNAL('renamed'),(self,oldName,name))

	name = property(getName,setName)
	
	def hasTracks(self):
		return len(self._trackItems)>0

	def addTrack(self,track):
		self.addTrackID(track.trackID)
		pass

	def addTrackID(self,trackID):
		self._trackItems.append({'Track ID':trackID})
		self.emit(PYSIGNAL('addedTrack'),(self,trackID))

	def addTrackIDs(self,trackIDs):
		for trackID in trackIDs:
			if not self.containsTrackID(trackID):
				self.addTrackID(trackID)

	def removeTrack(self,track):
		self.removeTrackID(track.trackID)
	
	def removeTrackID(self,trackID):
		self.emit(PYSIGNAL('aboutToRemoveTrack'),(self,trackID))
		for trackItem in self._trackItems:
			if trackItem['Track ID']==trackID:
				self._trackItems.remove(trackItem)
				break
		self.emit(PYSIGNAL('removedTrack'),(self,trackID))
	
	def removeTrackIDs(self,trackIDs):
		for trackID in trackIDs:
			self.removeTrackID(trackID)
		self.emit(PYSIGNAL('removedTracks'),(self,trackIDs))

	def containsTrack(self,track):
		return self.containsTrackID(track.trackID)

	def containsTrackID(self,trackID):
		for track in self._trackItems:
			if track['Track ID']==trackID:
				return True
		return False

	def trackAfter(self,track):
		trackIDs = self.getTrackIDs()
		try:
			index = trackIDs.index(track.trackID)
			index = index+1
			if index>=len(trackIDs):
				index = 0
			return self.trackWithTrackID(trackIDs[index])
		except:
			return None
	
	def trackBefore(self,track):
		trackIDs = self.getTrackIDs()
		try:
			index = trackIDs.index(track.trackID)
			index = index-1
			if index<0:
				index = len(trackIDs)-1
			return self.trackWithTrackID(trackIDs[index])
		except:
			return None

	def getTrackIDs(self):
		items = []
		for item in self._trackItems:
			items.append(item['Track ID'])
		return items
	trackIDs = property(getTrackIDs)

	def getTracks(self):
		items = []
		for trackID in self.trackIDs:
			items.append(self._library.trackWithTrackID(trackID))
		return items
	tracks = property(getTracks)

	def getEnabledTracks(self):
		tracks = self.getTracks()
		enabledTracks = []
		for track in tracks:
			if track.enabled:
				enabledTracks.append(track)
		return enabledTracks

	def trackWithTrackID(self,trackID):
		return self._library.trackWithTrackID(trackID)

	def isMaster(self):
		return self.master
	
	def isTrash(self):
		return self.trash

	def isPurchased(self):
		return self.purchased
	
	def canAcceptTrackDrags(self):
		return True
	
	def canAcceptFileDrags(self):
		return True

	def canInitiateDrag(self):
		return True

	def burn(self,burner,speed):
		tracks = self.getEnabledTracks()
		if len(tracks)>0:
			LMusicBurner.singleton().burnTracks(tracks,burner,speed)

	def exportPList(self):
		pList = {'Major Version':1,'Minor Version':0,'Application Version':'1.0'}
		playlists = []
		playlists.append(self.pList)
		pList['Playlists'] = playlists
		tracks = {}
		for item in self._trackItems:
			trackID = item['Track ID']
			track = self.trackWithTrackID(trackID)
			if track:
				tracks[str(trackID)] = track.pList
		pList['Tracks'] = tracks
		return [pList]

	def exportSongList(self):
		try: fileName = KFileDialog.getSaveFileName(None,"*.xml",None,"export playlist",i18n("Export Song List"))
		except: fileName = KFileDialog.getSaveFileName(None,"*.xml", None, i18n('Export Song List'))
		if fileName:
			self.exportTo(fileName)

	def exportTo(self,fileName):
		writer = PListWriter()
		writer.unparse(self.exportPList(),fileName)
	
	def indexOfTrackID(self,trackID):
		index = 0
		for item in self._trackItems:
			if item['Track ID']==trackID:
				return index
			index = index+1
		return -1
	
	def shuffle(self,trackIDs,insertPoint):
		for trackID in trackIDs:
			index = self.indexOfTrackID(trackID)
			if index<insertPoint:
				insertPoint = insertPoint-1
		#insertPoint -= len(trackIDs)
		self.removeTrackIDs(trackIDs)
		self.insertTrackIDsBefore(trackIDs,insertPoint)
		self.emit(PYSIGNAL("shuffled"),(self,None))

	def insertTrackIDsBefore(self,trackIDs,insertPoint):
		for trackID in trackIDs:
			self.insertTrackIDBefore(trackID,insertPoint)
			insertPoint = insertPoint+1
	
	def insertTrackIDBefore(self,trackID,insertPoint):
		self._trackItems.insert(insertPoint,{'Track ID':trackID})

	def burnPlaylistToDisc(self):
		# XXX DSM uncomment for testing
		#self.burn("0,0,0",0)
		#return
		if len(CDR.getBurnerDevices())>0:
			dialog = LBurnerSettingsDialog()
			ret = dialog.exec_loop()
			if ret==QDialog.Accepted:
				burner = dialog.getCurrentBurner()
				if burner:
					speed = dialog.getCurrentSpeed()
					#print "burning with",burner,"speed",speed
					self.burn(burner,speed)
		else:
			KMessageBox.error(None,"CD Burner not available","Cannot Burn")

	def trackIDsForArtists(self,artists):
##~ 		tracks = self.tracks
##~ 		trackIDs = []
##~ 		for track in tracks:
##~ 			if track and (unikode(track.artist) in artists):
##~ 				trackIDs.append(track.trackID)
##~ 		return trackIDs
		return self.trackIDsForArtistsAndAlbums(artists,None)

	def trackIDsForAlbums(self,albums):
		return self.trackIDsForArtistsAndAlbums(None,albums)
	
	def trackIDsForArtistsAndAlbums(self,artists,albums):
		trackIDs = []
		for track in self.tracks:
			artist = None
			if track.artist: artist = unikode(track.artist)
			album = None
			if track.album: album = unikode(track.album)			
			if artists==None or len(artists)==0 or (artist and artist in artists):
				if albums==None or len(albums)==0 or (album and album in albums):
					trackIDs.append(track.trackID)
		return trackIDs

	def getPList(self):
		playlist = {'Playlist ID':self.playlistID,'Name':self.name}
		items = []
		for item in self._trackItems:
			items.append({'Track ID':item['Track ID']})
		playlist['Playlist Items'] = items
		if self.master:
			playlist['Master'] = True
		if self.trash:
			playlist['Trash'] = True
		if self.purchased:
			playlist['Purchased Music'] = True
		if self.smart:
			playlist['Smart'] = True
		return playlist
	def setPList(self,pList):
		self.name = unikode(pList.get('Name','Unknown'))
		self.playlistID = pList.get('Playlist ID',0)
		a = pList['Playlist Items']
		for item in a:
			self.addTrackID(item['Track ID'])
		self.master = pList.get('Master',False)
		self.trash = pList.get('Trash',False)
		self.purchased = pList.get('Purchased Music',False)
		self.smart = pList.get('Smart',False)
	pList = property(getPList,setPList)

	def staticFromPList(pList):
		t = Playlist()
		t.pList = pList
		return t

	fromPList = staticmethod(staticFromPList)

if __name__=='__main__':
	p = Playlist()
	p.addTrackID(3)
	print p.trackIDs
	print p.pList
